@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.26.2.13 2010/04/13 04:59:12 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.37 2009/12/02 14:19:56 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2010	PgPool Global Development Group
+ * Copyright (c) 2003-2009	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -74,7 +74,7 @@ static void connection_count_down(void);
 /*
  * non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived
  */
-static int exit_request;
+volatile sig_atomic_t exit_request = 0;
 
 static int idle;		/* non 0 means this child is in idle state */
 static int accepted = 0;
@@ -101,6 +101,7 @@ void do_child(int unix_fd, int inet_fd)
 	struct timeval timeout;
 	static int connected;
 	int connections_count = 0;	/* used if child_max_connections > 0 */
+	int first_ready_for_query_received;		/* for master/slave mode */
 	int found;
 	char psbuf[NI_MAXHOST + 128];
 
@@ -173,11 +174,7 @@ void do_child(int unix_fd, int inet_fd)
 		StartupPacket *sp;
 
 		/* pgpool stop request already sent? */
-		if (exit_request)
-		{
-			die(0);
-			child_exit(0);
-		}
+		check_stop_request();
 
 		idle = 1;
 		accepted = 0;
@@ -292,6 +289,8 @@ void do_child(int unix_fd, int inet_fd)
 		 * we need to connect to the backend and send the startup packet.
 		 */
 
+		first_ready_for_query_received = 0;		/* for master/slave mode */
+
 		/* look for existing connection */
 		found = 0;
 		backend = pool_get_cp(sp->user, sp->database, sp->major, 1);
@@ -333,6 +332,13 @@ void do_child(int unix_fd, int inet_fd)
 				connection_count_down();
 				continue;
 			}
+
+			/* in master/slave mode, the first "ready for query"
+			 * packet should be treated as if we were not in the
+			 * mode
+			 */
+			if (MASTER_SLAVE)
+				first_ready_for_query_received = 1;
 		}
 
 		else
@@ -419,7 +425,7 @@ void do_child(int unix_fd, int inet_fd)
 		{
 			POOL_STATUS status;
 
-			status = pool_process_query(frontend, backend, 0);
+			status = pool_process_query(frontend, backend, 0, first_ready_for_query_received);
 
 			sp = MASTER_CONNECTION(backend)->sp;
 
@@ -448,7 +454,7 @@ void do_child(int unix_fd, int inet_fd)
 						POOL_STATUS status1;
 
 						/* send reset request to backend */
-						status1 = pool_process_query(frontend, backend, 1);
+						status1 = pool_process_query(frontend, backend, 1, 0);
 						pool_close(frontend);
 
 						/* if we detect errors on resetting connection, we need to discard
@@ -1085,23 +1091,22 @@ void cancel_request(CancelPacket *sp)
 	int i;
 	ConnectionInfo *c = NULL;
 	CancelPacket cp;
-	int loop_num = pool_config->num_init_children*pool_config->max_pool*MAX_NUM_BACKENDS;
 
 	pool_debug("Cancel request received");
 
 	/* look for cancel key from shmem info */
-	for (i=0;i<loop_num;i++)
+	for (i=0;i<pool_config->num_init_children*pool_config->max_pool;i++)
 	{
 		c = &con_info[i];
 
 		if (c->pid == sp->pid && c->key == sp->key)
 		{
 			pool_debug("found pid:%d key:%d i:%d",c->pid, c->key,i);
-			c = &con_info[i/(pool_config->max_pool * pool_config->max_pool)];
+			c = &con_info[i/pool_config->max_pool * pool_config->max_pool];
 			break;
 		}
 	}
-	if (i == loop_num)
+	if (i == pool_config->num_init_children*pool_config->max_pool)
 		return;	/* invalid key */
 
 	for (i=0;i<NUM_BACKENDS;i++,c++)
@@ -1202,13 +1207,13 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 }
 
 /*
- * signal handler for SIGINT and SIGQUUT
+ * signal handler for SIGTERM, SIGINT and SIGQUUT
  */
 static RETSIGTYPE die(int sig)
 {
-	exit_request = 1;
+	pool_debug("child received shutdown request signal %d", sig);
 
-	pool_debug("child receives shutdown request signal %d", sig);
+	exit_request = sig;
 
 	switch (sig)
 	{
@@ -1216,23 +1221,16 @@ static RETSIGTYPE die(int sig)
 			if (idle == 0)
 			{
 				pool_debug("child receives smart shutdown request but it's not in idle state");
-				return;
 			}
 			break;
 
 		case SIGINT:	/* fast shutdown */
 		case SIGQUIT:	/* immediate shutdown */
-			child_exit(0);
 			break;
 		default:
+			pool_error("die() received unknown signal: %d", sig);
 			break;
 	}
-
-	/*
-	 * child_exit() does this. So we don't need it.
-	 * send_frontend_exits();
-	 */
-	child_exit(0);
 }
 
 /*
@@ -1407,13 +1405,13 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	int len, len1;
 	int status;
 
-	cp = malloc(sizeof(POOL_CONNECTION_POOL_SLOT));
+	cp = malloc(sizeof(POOL_CONNECTION_POOL));
 	if (cp == NULL)
 	{
 		pool_error("make_persistent_db_connection: could not allocate memory");
 		return NULL;
 	}
-	memset(cp, 0, sizeof(POOL_CONNECTION_POOL_SLOT));
+	memset(cp, 0, sizeof(POOL_CONNECTION_POOL));
 
 	startup_packet = malloc(sizeof(*startup_packet));
 	if (startup_packet == NULL)
@@ -1853,3 +1851,22 @@ static RETSIGTYPE reload_config_handler(int sig)
 {
 	got_sighup = 1;
 }
+
+/*
+ * Exit myself if SIGTERM, SIGINT or SIGQUIT has been sent
+ */
+void check_stop_request(void)
+{
+    /*
+	 * If smart shutdown was requested but we are not in idle state,
+	 * do not exit
+	 */
+	if (exit_request == SIGTERM && idle == 0)
+		return;
+
+	if (exit_request)
+	{
+		reset_variables();
+		child_exit(0);
+	}
+}
