@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.45.2.10 2010/04/13 04:59:12 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.56 2009/12/06 12:55:08 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
@@ -84,6 +84,8 @@
 static void daemonize(void);
 static int read_pid_file(void);
 static void write_pid_file(void);
+static int read_status_file(void);
+static int write_statusd_file(void);
 static pid_t pcp_fork_a_child(int unix_fd, int inet_fd, char *pcp_conf_file);
 static pid_t fork_a_child(int unix_fd, int inet_fd, int id);
 static int create_unix_domain_socket(struct sockaddr_un un_addr_tmp);
@@ -163,6 +165,8 @@ static int pipe_fds[2]; /* for delivering signals */
 
 int my_proc_id;
 
+static BackendStatusRecord backend_rec;	/* Backend status record */
+
 int myargc;
 char **myargv;
 
@@ -356,6 +360,11 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	/*
+	 * Restore previous backend status if possible
+	 */
+	read_status_file();
+
 	/* clear cache */
 	if (clear_cache && pool_config->enable_query_cache && SYSDB_STATUS == CON_UP)
 	{
@@ -384,16 +393,7 @@ int main(int argc, char **argv)
 		inet_fd = create_inet_domain_socket(pool_config->listen_addresses, pool_config->port);
 	}
 
-	/*
-	 * con_info is a 3 dimension array: i corresponds to pgpool child
-	 * process, j corresponds to connection pool in each process and k
-	 * corresponds to backends in each connection pool.
-	 *
-	 * XXX: Before 2010/4/12 this was a 2 dimension array: i
-	 * corresponds to pgpool child process, j corresponds to
-	 * connection pool in each process. Of course this was wrong.
-	 */
-	size = pool_config->num_init_children * pool_config->max_pool * MAX_NUM_BACKENDS * sizeof(ConnectionInfo);
+	size = pool_config->num_init_children * pool_config->max_pool * sizeof(ConnectionInfo);
 	con_info = pool_shared_memory_create(size);
 	if (con_info == NULL)
 	{
@@ -412,7 +412,7 @@ int main(int argc, char **argv)
 	memset(pids, 0, size);
 	for (i = 0; i < pool_config->num_init_children; i++)
 	{
-		pids[i].connection_info = &con_info[i * pool_config->max_pool * MAX_NUM_BACKENDS];
+		pids[i].connection_info = &con_info[i * pool_config->max_pool];
 	}
 
 	/* create fail over/switch over event area */
@@ -782,6 +782,77 @@ static void write_pid_file(void)
 	}
 }
 
+/*
+* Read the status file
+*/
+static int read_status_file(void)
+{
+	FILE *fd;
+	char fnamebuf[POOLMAXPATHLEN];
+	int i;
+
+	snprintf(fnamebuf, sizeof(fnamebuf), "%s/%s", pool_config->logdir, STATUS_FILE_NAME);
+	fd = fopen(fnamebuf, "r");
+	if (!fd)
+	{
+		pool_log("Backend status file %s does not exist", fnamebuf);
+		return -1;
+	}
+	if (fread(&backend_rec, 1, sizeof(backend_rec), fd) <= 0)
+	{
+		pool_error("Could not read backend status file as %s. reason: %s",
+				   fnamebuf, strerror(errno));
+		fclose(fd);
+		return -1;
+	}
+	fclose(fd);
+
+	for (i=0;i< pool_config->backend_desc->num_backends;i++)
+	{
+		if (backend_rec.status[i] == CON_DOWN)
+			BACKEND_INFO(i).backend_status = CON_DOWN;
+		else
+			BACKEND_INFO(i).backend_status = CON_CONNECT_WAIT;
+	}
+
+	return 0;
+}
+
+/*
+* Write the pid file
+*/
+static int write_status_file(void)
+{
+	FILE *fd;
+	char fnamebuf[POOLMAXPATHLEN];
+	int i;
+
+	snprintf(fnamebuf, sizeof(fnamebuf), "%s/%s", pool_config->logdir, STATUS_FILE_NAME);
+	fd = fopen(fnamebuf, "w");
+	if (!fd)
+	{
+		pool_error("Could not open status file %s", fnamebuf);
+		return -1;
+	}
+
+	memset(&backend_rec, 0, sizeof(backend_rec));
+
+	for (i=0;i< pool_config->backend_desc->num_backends;i++)
+	{
+		backend_rec.status[i] = BACKEND_INFO(i).backend_status;
+	}
+
+	if (fwrite(&backend_rec, 1, sizeof(backend_rec), fd) <= 0)
+	{
+		pool_error("Could not write backend status file as %s. reason: %s",
+				   fnamebuf, strerror(errno));
+		fclose(fd);
+		return -1;
+	}
+	fclose(fd);
+	return 0;
+}
+
 /*
  * fork a child for PCP
  */
@@ -1002,6 +1073,8 @@ static void myexit(int code)
 	myunlink(pcp_un_addr.sun_path);
 	myunlink(pool_config->pid_file_name);
 
+	write_status_file();
+
 	pool_shmem_exit(code);
 	exit(code);
 }
