@@ -1,5 +1,21 @@
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pcp/pcp_proc_count.c,v 1.1.1.1 2006/09/08 03:36:50 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pcp/pcp_proc_count.c,v 1.4 2008/12/31 10:25:40 t-ishii Exp $
+ *
+ * pgpool: a language independent connection pool server for PostgreSQL 
+ * written by Tatsuo Ishii
+ *
+ * Copyright (c) 2003-2008	PgPool Global Development Group
+ *
+ * Permission to use, copy, modify, and distribute this software and
+ * its documentation for any purpose and without fee is hereby
+ * granted, provided that the above copyright notice appear in all
+ * copies and that both that copyright notice and this permission
+ * notice appear in supporting documentation, and that the name of the
+ * author not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior
+ * permission. The author makes no representations about the
+ * suitability of this software for any purpose.  It is provided "as
+ * is" without express or implied warranty.
  *
  * Client program to send "process count" command.
  */
@@ -7,6 +23,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "pcp.h"
 
@@ -23,53 +40,64 @@ main(int argc, char **argv)
 	char pass[MAX_USER_PASSWD_LEN];
 	int process_count;
 	int *process_list = NULL;
+	int ch;
 
-	if (argc == 2 && (strcmp(argv[1], "-h") == 0) )
-	{
-		usage();
-		exit(0);
+	while ((ch = getopt(argc, argv, "hd")) != -1) {
+		switch (ch) {
+		case 'd':
+			pcp_enable_debug();
+			break;
+
+		case 'h':
+		case '?':
+		default:
+			usage();
+			exit(0);
+		}
 	}
+	argc -= optind;
+	argv += optind;
 
-	if (argc != 6) {
+	if (argc != 5) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
 
-	timeout = atol(argv[1]);
+	timeout = atol(argv[0]);
 	if (timeout < 0) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
 
-	if (strlen(argv[2]) >= MAX_DB_HOST_NAMELEN) {
+	if (strlen(argv[1]) >= MAX_DB_HOST_NAMELEN) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
-	strcpy(host, argv[2]);
+	strcpy(host, argv[1]);
 
-	port = atoi(argv[3]);
+	port = atoi(argv[2]);
 	if (port <= 1024 || port > 65535) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
 
+	if (strlen(argv[3]) >= MAX_USER_PASSWD_LEN) {
+		errorcode = INVALERR;
+		pcp_errorstr(errorcode);
+		myexit(errorcode);
+	}
+	strcpy(user, argv[3]);
+
 	if (strlen(argv[4]) >= MAX_USER_PASSWD_LEN) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
-	strcpy(user, argv[4]);
-
-	if (strlen(argv[5]) >= MAX_USER_PASSWD_LEN) {
-		errorcode = INVALERR;
-		pcp_errorstr(errorcode);
-		myexit(errorcode);
-	}
-	strcpy(pass, argv[5]);
+	strcpy(pass, argv[4]);
 
 	pcp_set_timeout(timeout);
 
@@ -79,7 +107,7 @@ main(int argc, char **argv)
 		myexit(errorcode);
 	}
 
-	if ((process_list = pcp_process_count(&process_count)) < 0)
+	if ((process_list = pcp_process_count(&process_count)) == NULL)
 	{
 		pcp_errorstr(errorcode);
 		pcp_disconnect();
