@@ -11,7 +11,7 @@ public class CreateTempTableTest extends PgpoolTest {
 	    PreparedStatement pstmt = null;
 	    Statement stmt = null;
 
-	    connection.setAutoCommit(false);
+	    connection.setAutoCommit(true);
 
 	    pstmt = connection.prepareStatement("CREATE TEMP TABLE t1(i INTEGER)");
 	    pstmt.executeUpdate();
@@ -22,7 +22,7 @@ public class CreateTempTableTest extends PgpoolTest {
 	    pstmt.executeUpdate();
 	    pstmt.close();
 
-	    pstmt = connection.prepareStatement("UPDATE t1 SET i = ?");
+	    pstmt = connection.prepareStatement("/*NO LOAD BALANCE*/ UPDATE t1 SET i = ?");
 	    pstmt.setInt(1, 200);
 	    pstmt.executeUpdate();
 	    pstmt.close();
@@ -34,6 +34,16 @@ public class CreateTempTableTest extends PgpoolTest {
 		rs.close();
 		pstmt.close();
 
+	    pstmt = connection.prepareStatement("CREATE TABLE t2(i INTEGER, d TIMESTAMP DEFAULT CURRENT_TIMESTAMP)");
+	    pstmt.executeUpdate();
+
+	    pstmt = connection.prepareStatement("INSERT INTO t2(i) VALUES(?)");
+	    pstmt.setInt(1, 100);
+	    pstmt.executeUpdate();
+	    pstmt.setInt(1, 200);
+	    pstmt.executeUpdate();
+	    pstmt.close();
+
 	    connection.commit();
 	}
 	finally {
